@@ -117,7 +117,7 @@ class SubTensor(np.ndarray):
     def random(cls, shape, q_labels=None, dtype=float):
         if dtype == float:
             obj = np.random.random(shape)
-        elif dtype == complex:
+        elif dtype == complex or dtype == np.complex128:
             obj = np.random.random(shape) + np.random.random(shape) * 1j
         else:
             return NotImplementedError('dtype %r not supported!' % dtype)
@@ -432,7 +432,7 @@ _numpy_func_impls = _sparse_tensor_numpy_func_impls
 class SparseTensor(NDArrayOperatorsMixin):
     """
     block-sparse tensor.
-    Represented as a list of :class:`SubTesnor`.
+    Represented as a list of :class:`SubTensor`.
     """
 
     def __init__(self, blocks=None):
@@ -672,7 +672,8 @@ class SparseTensor(NDArrayOperatorsMixin):
         """Kron product of quantum numbers along selected indices, for fusing purpose."""
         idxs = np.array([i if i >= 0 else self.ndim +
                          i for i in idxs], dtype=np.int32)
-        return BondFusingInfo.tensor_product(*np.array(self.infos)[idxs], pattern=pattern)
+        return BondFusingInfo.tensor_product(*np.array(list(self.infos) + [[]],
+            dtype=object)[idxs], pattern=pattern)
 
     @staticmethod
     def _unfuse(a, i, info):
@@ -928,6 +929,10 @@ class SparseTensor(NDArrayOperatorsMixin):
             infos = (abi[0] + bbi[0], abi[-1] + bbi[-1])
 
         lb, rb = infos[0], infos[-1]
+        if a.dtype == np.complex128 or b.dtype == np.complex128:
+            dtype = np.complex128
+        else:
+            dtype = a.dtype
 
         sub_mp = {}
         # find required new blocks and their shapes
@@ -939,7 +944,7 @@ class SparseTensor(NDArrayOperatorsMixin):
                 mshape[0] = lb[qs[0]]
                 mshape[-1] = rb[qs[-1]]
                 sub_mp[qs] = SubTensor.zeros(shape=tuple(
-                    mshape), q_labels=qs, dtype=a.dtype)
+                    mshape), q_labels=qs, dtype=dtype)
         # copy block self.blocks to smaller index in new block
         for block in a.blocks:
             qs = block.q_labels
@@ -1544,7 +1549,8 @@ class FermionTensor(NDArrayOperatorsMixin):
     def kron_product_info(self, *idxs, pattern=None):
         idxs = np.array([i if i >= 0 else self.ndim +
                          i for i in idxs], dtype=np.int32)
-        return BondFusingInfo.tensor_product(*np.array(self.infos)[idxs], pattern=pattern)
+        return BondFusingInfo.tensor_product(*np.array(list(self.infos) + [[]],
+            dtype=object)[idxs], pattern=pattern)
 
     @staticmethod
     def _unfuse(a, i, info):
