@@ -153,7 +153,7 @@ class MPS(NDArrayOperatorsMixin):
         const : float
             Constant term.
         opts : dict or None
-            Options indicating how bond dimension trunctation
+            Options indicating how bond dimension truncation
             should be done after MPO @ MPS, etc.
             Possible options are: max_bond_dim, cutoff, max_dw, norm_cutoff
         dq : SZ
@@ -229,6 +229,12 @@ class MPS(NDArrayOperatorsMixin):
                     (info.left_dims[i], info.basis[i], info.left_dims[i + 1]),
                     dtype=dtype) * (high - low) + low
         return MPS(tensors=tensors, opts=opts)
+    
+    def fix_pattern(self, pattern=None):
+        dq = None
+        for i in range(self.n_sites):
+            self.tensors[i], dq = self.tensors[i].fix_pattern(pattern=pattern, dq=dq)
+        return dq
 
     def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
         if ufunc in _mps_numpy_func_impls:
@@ -443,8 +449,8 @@ class MPS(NDArrayOperatorsMixin):
     @implements(np.linalg.norm)
     def _norm(x):
         d = np.conj(x).dot(x)
-        assert abs(d.imag) < 1E-14
-        return np.sqrt(abs(d.real) if abs(d.real) < 1E-14 else d.real)
+        assert abs(d.imag) / abs(d.real) < 1E-10
+        return np.sqrt(abs(d.real) if abs(d.real) < 1E-10 else d.real)
 
     def norm(self):
         return np.linalg.norm(self)
@@ -593,6 +599,54 @@ class MPS(NDArrayOperatorsMixin):
                 tensors[it] = ts.to_sparse(dq=None if it == 0 else self.dq)
         return MPS(tensors=tensors, const=self.const, opts=self.opts, dq=self.dq)
 
+    @staticmethod
+    def _to_symbolic(a):
+        return a.to_sparse()
+
+    def to_symbolic(self):
+        tensors = [None] * len(self.tensors)
+        for it, ts in enumerate(self.tensors):
+            if isinstance(ts, FlatFermionTensor):
+                tensors[it] = ts.to_fermion()
+            else:
+                tensors[it] = ts
+        infos = [t.infos for t in tensors]
+        bonds = []
+        bonds.append(infos[0][0])
+        for i in range(self.n_sites - 1):
+            bonds.append(infos[i + 1][0] | infos[i][-1])
+        bonds.append(infos[-1][-1])
+        for it, ts in enumerate(tensors):
+            pos = -1 if it == self.n_sites - 1 else int(it != 0)
+            if isinstance(ts, SymbolicSparseTensor):
+                tensors[it] = ts
+            if isinstance(ts, FlatSparseTensor):
+                assert False
+            elif isinstance(ts, FlatFermionTensor):
+                tensors[it] = SymbolicSparseTensor.from_sparse(ts.to_fermion(),
+                    pos=pos, infos=bonds[it:it + 2], dq=None if it == 0 else self.dq)
+            else:
+                tensors[it] = SymbolicSparseTensor.from_sparse(ts,
+                    pos=pos, infos=bonds[it:it + 2], dq=None if it == 0 else self.dq)
+        return MPS(tensors=tensors, const=self.const, opts=self.opts, dq=self.dq)
+
+    @staticmethod
+    def _to_non_flat(a):
+        return a.to_non_flat()
+
+    def to_non_flat(self):
+        tensors = [None] * len(self.tensors)
+        for it, ts in enumerate(self.tensors):
+            if isinstance(ts, FlatSparseTensor):
+                tensors[it] = ts.to_sparse()
+            elif isinstance(ts, FlatFermionTensor):
+                tensors[it] = ts.to_fermion()
+            elif isinstance(ts, SparseTensor) or isinstance(ts, FermionTensor):
+                tensors[it] = ts
+            else:
+                tensors[it] = ts.to_sparse()
+        return MPS(tensors=tensors, const=self.const, opts=self.opts, dq=self.dq)
+
     @staticmethod
     def _to_flat(a):
         return a.to_flat()
